@@ -24,6 +24,7 @@
 #define __G_MOUNT_TRACKER_H__
 
 #include <glib-object.h>
+#include <gio/gio.h>
 #include <gmountspec.h>
 
 G_BEGIN_DECLS
@@ -42,12 +43,14 @@ typedef struct {
   volatile int ref_count;
   char *display_name;
   char *stable_name;
-  char *icon;
+  char *x_content_types;
+  GIcon *icon;
   char *dbus_id;
   char *object_path;
   gboolean user_visible;
   char *prefered_filename_encoding; /* NULL -> UTF8 */
   char *fuse_mountpoint;
+  char *default_location;
   GMountSpec *mount_spec;
 } GMountInfo;
 
