@@ -28,7 +28,7 @@
 #include <locale.h>
 #include <stdlib.h>
 #include <dbus/dbus.h>
-#include "gdbusutils.h"
+#include "gvfsdbusutils.h"
 #include "metatree.h"
 #include "gvfsdaemonprotocol.h"
 
@@ -321,7 +321,7 @@ metadata_get (const char *treefile,
 	      DBusError *derror)
 {
   TreeInfo *info;
-  const char *key;
+  char *key;
   DBusMessage *reply;
   DBusMessageIter reply_iter;
   GPtrArray *keys;
